-- YGREC8/VHDL/ALU8/ROP2_ASIC.vhdl
-- created lun. déc.  30 19:48:33 CET 2019 by whygee@f-cpu.org
-- version mer. janv.  1 19:56:27 CET 2020
--
-- Released under the GNU AGPLv3 license
--
-- This is the boolean/logic unit, which shares gates with the adder.
-- This version is optimised for ASIC (or discrete) implementations.
--
--        NEG  PASS_en AND_en XOR_en OrXor_en 
-- AND     0       0      0      0      0   
-- OR      0       0      1      1      1
-- XOR     0       0      1      0      1
-- ANDN    1       0      0      0      0
-- SUB     1       0      1      1      0
-- ADD     0       0      1      1      0
-- PASS    x       1      1      1      0


Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
Library SLV;
    use SLV.SLV_def.all;
Library proasic3;
    use proasic3.all;


entity ROP2 is
  port(
    NEG, PASS_en, AND_en, XOR_en, OrXor_en : in SL;
    SRI, SND : in SLV8;
    ROP2, nP, nG, G, P, X : inout SLV8);
end ROP2;

architecture ASIC of ROP2 is
  Signal XOR_SND, XOR_sel, ORXOR_sel, AND_sel, PASS_sel : SLV8;

begin
  gen: for I in 0 to 7 generate
    --1
    rop2neg:   entity XOR2  port map(A=>NEG,    B=>SND(I),  Y=>XOR_SND(I)); 
    rop2Pass:  entity NAND2 port map(A=>SRI(I), B=>PASS_en, Y=>PASS_sel(I));
    --2
    rop2P:     entity NOR2  port map(A=>XOR_SND(I), B=>SRI(I), Y=>nP(I));
    rop2G:     entity NAND2 port map(A=>XOR_SND(I), B=>SRI(I), Y=>nG(I));
    -- 3 : buffers of strength >4
    rop2BufP:  entity INV   port map(A=>nP(I), Y=>P(I));
    rop2BufG:  entity INV   port map(A=>nG(I), Y=>G(I));
    -- 4
    rop2X:     entity NOR2  port map(A=>G(I), B=>nP(I), Y=>X(I));
    rop2Gen:   entity NAND2 port map(A=>G(I), B=>XOR_en,   Y=>XOR_sel(I));
    --
    rop2or:    entity NAND3 port map(A=>XOR_sel(I), B=>OrXor_en, C=>P(I), Y=>ORXOR_sel(I));
    rop2and:   entity OR2   port map(A=>XOR_sel(I), B=>AND_en,   Y=>AND_sel(I));
    --
    rop2out:   entity NAND3 port map(A=>ORXOR_sel(I), B=>AND_sel(I), C=>PASS_sel(I), Y=>ROP2(I));
  end generate gen;
end ASIC;
