-- YGREC8/VHDL/ALU8/test_decoder.vhdl
-- created dim. janv.  5 17:54:37 CET 2020 by whygee@f-cpu.org
--
-- Released under the GNU AGPLv3 license
--
-- Just exercises the whole ALU with some simple vectors.

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;
    use work.ygrec8_asm.all;
Library SLV;
    use SLV.SLV_def.all;
Library YGREC8;
    use YGREC8.ygrec8_def.all;
Library conversions;
    use conversions.conversions.all;

entity test_decoder is
end test_decoder;

architecture test of test_decoder is
  signal Inst : SLV16;
  signal SND, SRI, ROP, SUM, RES : SLV8;
  signal Cin, Cout, NEG, Pass_en, AND_en, XOR_en, OrXor_en, CLA_en, CMPS : SL;
begin

  -- connect the decoder to the ALU:

deco: entity DEC_ALU8(LUT) port map (
--  deco: entity DEC_ALU8(equations) port map (
        Inst => Inst,

         Cin => Cin,
         NEG => NEG,
     PASS_en => PASS_en,
      AND_en => AND_en,
      XOR_en => XOR_en,
    OrXor_en => OrXor_en,
      CLA_en => CLA_en,
        CMPS => CMPS);
 
  alu: entity ALU8 port map (
         Cin => Cin,
         NEG => NEG,
        CMPS => CMPS,
     PASS_en => PASS_en,
      AND_en => AND_en,
      XOR_en => XOR_en,
    OrXor_en => OrXor_en,
-- not yet   CLA_en => CLA_en,

         SRI => SRI,
         SND => SND,
         ROP => ROP,
         SUM => SUM,
        Cout => Cout );

  -- mux !
  RES <= SUM when CLA_en='1' else ROP;

---------------------------------------------------------------

  process is

    procedure display_outputs(s: string; exp : SLV8) is
      variable ctx: string(1 to 5) := "oops ";
    begin
      inst <= Assemble(s, ctx);

      wait for 1 ms;

      report s
       & " -> inst=" & SLV_to_bin(inst)
           & " NEG=" & std_logic'image(NEG)(2)
          & " PASS=" & std_logic'image(PASS_en)(2)
           & " AND=" & std_logic'image(AND_en)(2)
           & " XOR=" & std_logic'image(XOR_en)(2)
         & " OrXor=" & std_logic'image(OrXor_en)(2)
          & " CMPS=" & std_logic'image(CMPS)(2)
           & " cla=" & std_logic'image(CLA_en)(2)
           & " SRI=" & SLV_to_bin(SRI)
           & " SND=" & SLV_to_bin(SND)
--        & " - ROP2=" & SLV_to_bin(ROP)
--         & " - SUM=" & SLV_to_bin(SUM)
          & " Cout=" & std_logic'image(Cout)(2)
         & " - RES=" & SLV_to_bin(RES);

      assert exp = RES
         report "RES " & SLV_to_bin(RES) & " /= " & SLV_to_bin(exp)
         severity warning;
--failure; -- warning;
    end procedure;

    procedure vectors(j, k : integer) is
      variable sum, sum1, diff : SLV9;
      variable A, B : SLV8;
    begin
      A := std_logic_vector(to_unsigned(j, 8));
      B := std_logic_vector(to_unsigned(k, 8));
      SRI <= A;
      SND <= B;
      sum := std_logic_vector(to_unsigned(j+k,   9));
      sum1:= std_logic_vector(to_unsigned(j+k+1, 9));
      diff:= std_logic_vector(  to_signed(j-k,   9));

--      expected := t(7 downto 0);
--      if t /= ('0' & Cout & SUM) then ShowSignals(7); end if;
--       sum := SLV8(
--      diff :=
--       sum := "00000000";
--      diff := "00000000";
      display_outputs("OR    D1 D1", A  or      B);
      display_outputs("XOR   D1 D1", A xor      B);
      display_outputs("AND   D1 D1", A and      B);
      display_outputs("ANDN  D1 D1", A and (not B));
      display_outputs("CMPU  D1 D1", diff(7 downto 0));
        assert diff(8) = Cout report "Wrong Cout !" severity warning;
      display_outputs("CMPS  D1 D1", diff(7 downto 0));
        assert diff(8) = Cout report "Wrong Cout !" severity  warning;
      display_outputs("SUB   D1 D1", diff(7 downto 0));
        assert diff(8) = Cout report "Wrong Cout !" severity warning;
      display_outputs("ADD   D1 D1", sum(7 downto 0)); -- check no Cin
        assert sum(8) = Cout report "Wrong Cout !" severity warning;
      display_outputs("ADD  127 D1", sum(7 downto 0)); -- check no Cin
        assert sum(8) = Cout report "Wrong Cout !" severity warning;
      display_outputs( "ADD    1 D1", sum1(7 downto 0)); -- force Cin
        assert sum1(8) = Cout report "Wrong Cout !" severity warning;
      display_outputs( "ADD   -1 D1", sum(7 downto 0)); -- check no Cin
        assert sum(8) = Cout report "Wrong Cout !" severity warning;
      display_outputs( "SET   D1 D1", A);
      display_outputs( "CALL  D1 D1", A);
      display_outputs( "SH    D1 D1", "00000000");
      display_outputs( "RC    D1 D1", "00000000");
      display_outputs( "LDCL  D1 D1", A);
      display_outputs( "IN     1 D1", "00000000");
      display_outputs( "OUT    1 D1", "00000000");
      display_outputs( "INV        ", "00000000");
    end vectors;

  begin
    report "Starting test of the decoder+ALU";
    vectors(10, 12);
    vectors(12, 10);

--    SRI <= "00001100";  SND <= "00001010"; vectors;
--    SRI <= "00001010";  SND <= "00001100"; vectors;

--    display_outputs('0', '0', '0', '0', '1', "OR   ");
--    display_outputs('0', '0', '1', '1', '1', "XOR  ");
--    display_outputs('0', '0', '0', '1', '0', "AND  ");
--    display_outputs('1', '0', '0', '1', '0', "ANDN ");
--    display_outputs('1', '0', '1', '0', '0', "SUB  ");
--    display_outputs('1', '0', '1', '1', '0', "SUB  ");
--    display_outputs('0', '0', '1', '0', '0', "ADD  ");
--    display_outputs('0', '0', '1', '1', '0', "ADD  ");
--    display_outputs('0', '1', '1', '1', '0', "PASS1");
--    display_outputs('0', '1', '1', '0', '0', "PASS2");
--    display_outputs('1', '1', '1', '1', '0', "PASS3");
--    display_outputs('1', '1', '1', '0', '0', "PASS4");
--    display_outputs('0', '0', '0', '0', '0', "clear");
--    display_outputs('1', '0', '0', '0', '0', "clear");
    
    wait;
  end process;

end test;
