-- file YGREC8/VHDL/FSM/test_FSM.vhdl
-- created ven. janv. 17 22:52:11 CET 2020 by whygee@f-cpu.org
--
-- Released under the GNU AGPLv3 license

Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
Library SLV;
    use SLV.SLV_def.all;
Library YGREC8;
    use YGREC8.ygrec8_def.all;
Library conversions;
    use conversions.conversions.all;


entity test_FSM is
  -- top level -> nothing
end test_FSM;

architecture behavioural of test_FSM is

  signal Clk, Reset, CmdRdy, SND_PC, Opc_LDCx,
      Opc_INV, PC_ov, Decode : SL := '0';
  signal Command: tControlCommand;
  signal Phase  : tInstrPhase;
  signal Status : tControlState;
begin

  efsm: entity Y8_FSM port map(
    Clk => Clk,
    Reset => Reset,
    CmdRdy => CmdRdy,
    SND_PC => SND_PC,
    Opc_LDCx => Opc_LDCx,
    Opc_INV => Opc_INV,
    PC_ov => PC_ov,
    Command => Command,
    Status => Status,
    Phase => Phase,
    Decode => Decode);

  process is
    variable RunStep : SL;

    procedure cycle is
    begin
      wait for 500 us;
      Clk <= '1' ;
      wait for 500 us;
      Clk <= '0' ;
    end cycle;

    procedure exp(C : tControlState; P : tInstrPhase) is
    begin
      assert Status=C report "State FSM should have been " & StatusString(C) severity failure;
      assert Phase=P  report "Phase FSM should have been " & PhaseString(P)  severity failure;
    end exp;

  begin
report "* RESET";
    wait for 1 sec; wait for 500 us;
    exp(S_RESET, P_IDLE);

report "* Releasing RESET";
    Reset <= '1' ; -- release reset 
    cycle;
    cycle;
    exp(S_LOAD, P_IDLE);
    cycle;
    cycle; -- we'll have to bound/ensure/check the min-max transition times

report "* End load";
    PC_ov <= '1'; -- end the load
    cycle;
    PC_ov <= '0';
    exp(S_STOP, P_IDLE);
    cycle;
    exp(S_RUN, P_IDLE);
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    cycle; -- now it should run.
    cycle;
    exp(S_RUN, P_INST);

report "* Request step";
    Command <= C_STEP; -- try to Step (transition=>stop)
    CmdRdy <= '1';
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    exp(S_STEP, P_INST);
    cycle;
    exp(S_STOP, P_INST);
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;
    cycle;  -- CmdRdy can't be shorter than 1 core cycle.
    cycle;  -- how can the synchro work ?
    cycle;
    CmdRdy <= '0';
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;

report "* Request step again";
    CmdRdy <= '1';
    cycle;
    exp(S_STOP, P_IDLE);  -- yes there is a 2-cycles latency
    cycle;
    exp(S_STEP, P_IDLE);
    cycle;
    exp(S_STOP, P_INST);
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;
    cycle;
    CmdRdy <= '0';
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;

report "* Run again";
    Command <= C_START;
    CmdRdy <= '1';
    cycle;
    exp(S_STOP, P_IDLE);  -- yes there is a 2-cycles latency
    cycle;
    exp(S_RUN, P_IDLE);
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    cycle;
    CmdRdy <= '0';
    cycle;
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    cycle;

report "* End run by INV";
    Opc_INV <= '1'; -- throw a hammer in the air
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;
    cycle;
    Opc_INV <= '0';
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;  -- expect the hammer to stay in the air
    cycle;
    cycle;
    exp(S_STOP, P_IDLE);

report "* Run again (2)";
    Command <= C_START;
    CmdRdy <= '1';
    cycle;
    exp(S_STOP, P_IDLE);  -- yes there is a 2-cycles latency
    cycle;
    exp(S_RUN, P_IDLE);
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    CmdRdy <= '0';
    cycle;
    exp(S_RUN, P_INST);

report "* Force RESET";
    Command <= C_RESET;
    CmdRdy <= '1';
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    CmdRdy <= '0';
    exp(S_RESET, P_INST);
    cycle;
    exp(S_RESET, P_IDLE);
    cycle;

report "* Back to STOP";
    Command <= C_STOP;
    CmdRdy <= '1';
    cycle;
    CmdRdy <= '0';
    exp(S_RESET, P_IDLE);
    cycle;
    exp(S_LOAD, P_IDLE);
    cycle;
    PC_ov <= '1'; -- end the load
    cycle;
    exp(S_STOP, P_IDLE);
    PC_ov <= '0';
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;
    cycle;
    cycle;
    cycle;
    exp(S_STOP, P_IDLE);

report "* Run again (3)";
    Command <= C_START;
    CmdRdy <= '1';
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;
    exp(S_RUN, P_IDLE);
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    CmdRdy <= '0';
    cycle;
    exp(S_RUN, P_INST);

report "* Trying LDCx";
    Opc_LDCx <= '1';
    cycle;
    exp(S_RUN, P_LDCX);
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    exp(S_RUN, P_LDCX);
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    exp(S_RUN, P_LDCX);

report "* Now writing to PC";
    SND_PC <= '1';
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    exp(S_RUN, P_LDCX);
    cycle;
    exp(S_RUN, P_WrPC);
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    exp(S_RUN, P_LDCX);
    cycle;
    exp(S_RUN, P_WrPC);

report "* Now not LDCx";
    Opc_LDCx <= '0';
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    exp(S_RUN, P_WrPC);
    cycle;
    exp(S_RUN, P_INST);
    cycle;
    exp(S_RUN, P_WrPC);


report "* Back to STOP after WrPC";
    Command <= C_STOP;
    CmdRdy <= '1';
    cycle;
    exp(S_RUN, P_INST);
    CmdRdy <= '0';
    cycle;
    exp(S_STOP, P_WrPC);
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;
    exp(S_STOP, P_IDLE);
    cycle;
    exp(S_STOP, P_IDLE);

    wait;
  end process;

end behavioural;

