-- YGREC8/VHDL/TAP/Gray/Gray6s_behavioral.vhdl
-- created ven. juil. 17 19:02:35 CEST 2020 by whygee@f-cpu.org
-- version mar. août 11 10:21:45 CEST 2020 : from 6 to 7s
-- version mer. août 12 02:54:56 CEST 2020 : added FB & Less4 outputs
-- version ven. août 14 17:49:32 CEST 2020 : back to 6 bits
-- version mer. août 19 21:18:52 CEST 2020 : more signals output for the Selector (OV1)
-- version mar. août 25 05:47:32 CEST 2020 : abstracted the RS FF a bit...
--
-- Released under the GNU AGPLv3 license
--
-- This is a trivial Gray code counter. This is only used for
-- behavioral simulations because the output is potentially
-- glitchy and would require another 2 levels of DFF.
-- The RESET method is changed as well...

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library SLV;
    use SLV.SLV_def.all;

entity Gray6s is
  port(
    CLK, WR : in  SL;
    SAT, ENABLE, preFB, FB, Less4, OV1 : out SL;
    G6 : out SLV6);
end Gray6s;

architecture behavioral of Gray6s is
  signal bin : unsigned(5 downto 0) := "000000";
  signal   t : unsigned(6 downto 0) := "0000000";
  signal tG6, D, Q : SLV6 := "000000";
  signal last_WR, Diff, tFB, tPreFB,
    tSat, lSat, pLock : SL := '0';

begin
  -- gather all the DFFs:
  process (CLK) is
  begin
    if rising_edge(CLK) then
      last_WR <= WR;
      tFB <= tPreFB;
      lSat <= tSat or SL(t(6));
      Q <= SLV6(t(Q'range));
    end if;
  end process;

  -- the SR latch:
  Diff <= WR xor last_WR;
  pLock <= '0' when Diff='1'
      else '1' when CLK='1';
  ENABLE <= pLock;

  -- the counter :
  tG6 <= Q and (tG6'range=>pLock);
  t <= unsigned('0' & tG6) + 1;
  -- the Gray code :
  G6 <= tG6(5) & ( tG6(5 downto 1)
               xor tG6(4 downto 0));

  -- saturation ?
  tSat <= lSat and pLock;
  SAT <= tSat;

  -- Full Byte flag :
  tPreFB <= '1' when tG6(2 downto 0)="111"
      else '0';
  preFB <= tPreFB;
  FB <= tFB and pLock;

  -- size predecode :
  Less4 <= '1' when tSat='0'
              and tG6(5)='0'
              and WR='0'
      else '0';

  OV1 <= '1' when tG6(0)='1' and tG6(1)='1' else '0';

end behavioral;
