-- YGREC8/VHDL/TAP/Gray/Gray7s_tiles.vhdl
-- created jeu. aout 13 00:39:31 CEST 2020 by whygee@f-cpu.org
-- version ven. août 14 19:03:26 CEST 2020 : back to 6 bits
-- version mer. août 19 21:18:52 CEST 2020 : more signals output for the Selector (OV1)
-- version mar. août 25 05:47:32 CEST 2020 : changed the RS flip-flop
--
-- Released under the GNU AGPLv3 license
--
-- This is a rather sophisticated Gray code counter with saturation,
-- resynchronisation to dual-edge RESET and some predecoding,
-- using only ASIC/ProASIC gates. See the PNG.
-- Some recurring code has been put in for..generate loops
-- to keep the source shorter and cleaner, unlike the previous
-- version (with some special cases handled by manual assignations)
--
-- 38 gates :-)

Library ieee;
    use ieee.std_logic_1164.all;
Library SLV;
    use SLV.SLV_def.all;
Library proasic3;
    use proasic3.all;

entity Gray6s is
  port(
    CLK, WR : in  SL := '0';
    SAT, ENABLE, preFB, FB, Less4, OV1 : out SL;
    G6 : out SLV6);
end Gray6s;

architecture tiles of Gray6s is

  -- There are 9 DFF :
  -- 0 to 6 are gray counters
  -- 6 is saturation
  -- 7 is FB
  -- 8 is oldWR
  signal DFFi, DFFo : SLV9 := "000000000";  -- outputs of the DFFs
  signal DFFa : SLV8 := "00000000";  -- result of the AND
  signal OV : std_logic_vector(3 downto 1) := "000";
  signal DIR : SLV2 := "00";
  signal OVl : std_logic_vector(2 downto 1) := "00";
  signal b5i, Diff, EN, XORp, XORn, XORM : SL := '0';
--  signal nLock : SL := '1';

  component AND2  port(A, B    : in std_logic; Y : out std_logic); end component;
  component AO1   port(A, B, C : in std_logic; Y : out std_logic); end component;
  component DFN1  port(D,  CLK : in std_logic; Q : out std_logic); end component;
  component INV   port(A       : in std_logic; Y : out std_logic); end component;
  component MX2   port(A, B, S : in std_logic; Y : out std_logic); end component;
  component NOR2  port(A, B    : in std_logic; Y : out std_logic); end component;
  component NOR3  port(A, B, C : in std_logic; Y : out std_logic); end component;
  component R1S1  port(S, R    : in std_logic; Y : out std_logic); end component;
  component XOR2  port(A, B    : in std_logic; Y : out std_logic); end component;
  component XOR3  port(A, B, C : in std_logic; Y : out std_logic); end component;
  component XNOR2 port(A, B    : in std_logic; Y : out std_logic); end component;
begin

  -- Some repeated instances:
  ldff: for i in DFFi'range generate  -- 8 downto 0
    dff: DFN1 port map(D=>DFFi(i), CLK=>CLK, Q=>DFFo(i));
  end generate;

  land: for i in DFFa'range generate -- 7 downto 0
    enab: AND2 port map(A=>DFFo(i), B=>EN, Y=>DFFa(i));
  end generate;

  -- resynch:
  DFFi(8) <= WR;
  rsx: XOR2 port map(A=>WR, B=>DFFo(8), Y=>Diff);
  rsy: R1S1 port map(S=>CLK, R=>Diff, Y=>EN);
--  rsy: NOR2 port map(A=>Diff, B=>nLock, Y=>EN);
--  rsz: NOR2 port map(A=>EN, B=>CLK, Y=>nLock);
  ENABLE <=EN;

  -- Lower bits :
  xn_0:  XNOR2 port map(A=>DIR(0),  B=>DFFa(1), Y=>DFFi(0));
  no_0:   NOR2 port map(A=>DFFi(0), B=>DFFa(0), Y=>OV(1));
  xm_0:   XOR3 port map(A=>DIR(0),  B=>DFFa(0), C=>OV(1), Y=>DFFi(1));
  OV1 <= OV(1);

  -- Middle bits :
  mx_2:   MX2  port map(A=>DFFa(2), B=>XORn,    S=>OV(1), Y=>DFFi(2));
  mx_3:   MX2  port map(A=>DFFa(3), B=>XORM,    S=>OV(1), Y=>DFFi(3));

  xp_1:  XOR2  port map(A=>DIR(1),  B=>DFFa(2), Y=>XORp);
  xn_1: XNOR2  port map(A=>DIR(1),  B=>DFFa(3), Y=>XORn);
  no_1:  NOR2  port map(A=>XORn,    B=>DFFa(2), Y=>OVl(1));
  xm_1:  XOR2  port map(A=>OVl(1),  B=>XORp,    Y=>XORM);

  dir_0: XOR2  port map(A=>DFFa(3), B=>XORp,    Y=>DIR(0));
  ov2:   AND2  port map(A=>OVl(1),  B=>OV(1),   Y=>OV(2));

  -- High bits :
  mx_4:   MX2  port map(A=>DFFa(4), B=>b5i,     S=>OV(2), Y=>DFFi(4));
  mx_5:   MX2  port map(A=>DFFa(5), B=>DFFa(4), S=>OV(2), Y=>DFFi(5));

  iv2:    INV  port map(A=>DFFa(5), Y=>b5i);
  no_2:   NOR2 port map(A=>b5i,     B=>DFFa(4), Y=>OVl(2));
  dir1:   XOR2 port map(A=>DFFa(5), B=>DFFa(4), Y=>DIR(1));

  -- the Saturation:
  ao:     AO1  port map(A=>OV(2),   B=>OVl(2), C=>DFFa(6), Y=>DFFi(6));
  SAT <= DFFa(6);

  -- Less than 4 bytes :
  l4: NOR3 port map(A=>DFFa(5), B=>DFFa(6), C=>WR, Y=>Less4);

  -- Full Byte flags :
  aFB: AND2 port map(A=>DIR(0), B=>OV(1), Y=>DFFi(7));
  preFB <= DFFi(7);
     FB <= DFFa(7);

  G6 <= DFFa(G6'range);
end tiles;
