-- file YGREC8/VHDL/TAP/Gray/test_Gray7s.vhdl
-- created ven. juil. 17 19:48:28 CEST 2020
-- version mar. août 11 10:51:08 CEST 2020 adapted from test_Gray6.vhdl to test_Gray7s.vhdl
-- version ven. août 14 17:49:32 CEST 2020 : back to 6 bits
--
-- Released under the GNU AGPLv3 license

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library SLV;
    use SLV.SLV_def.all;
Library conversions;
    use conversions.conversions.all;
Library work;
    use work.all;

entity test_Gray6s is
  generic (  maxcount: integer := 300  );
  -- top level -> nothing
end test_Gray6s;

architecture behavioural of test_Gray6s is
  signal Clk, WR, SAT, Less4, ENABLE, OV1 : SL := '0';
  signal G6 : SLV6;
begin

  dut: entity Gray6s port map(
      Clk => Clk,
       WR => WR,
      SAT => SAT,
      OV1 => OV1,
    Less4 => Less4,
   ENABLE => ENABLE,
       G6 => G6 );

  process is
    variable count : integer;
    variable s : SL;

    procedure cycle is
    begin
      wait for 500 us;
      Clk <= '1' ;
      wait for 500 us;
      Clk <= '0' ;
    end cycle;

    procedure show is
      variable v, l4 : SL;
      variable t : SLV6;
      variable j : integer;
    begin
      t := G6 xor ("0000" & G6(5 downto 4)); --  num ^= num >>  4;
      t :=  t xor (  "00" &  t(5 downto 2)); --  num ^= num >>  2;
      t :=  t xor (   '0' &  t(5 downto 1)); --  num ^= num >>  1;
      j :=  safe_to_integer(t);

      s := '0';
      if count >= 64 then
        s := '1';
      end if;

      v := '0';
      if (count mod 4) = 3 then
        v := '1';
      end if;

      l4 := '0';
      if count < 32 and WR/='1' then
        l4 := '1';
      end if;
-- SAT ?

      if maxcount /= 300 then
        report  integer'image(count)
            & ":  " & SLV_to_bin(G6)
            & " = " & SLV_to_bin(t)
            & " = " & integer'image(j)
            & "  EN=" & SL'image(ENABLE)
            & "  L4=" & SL'image(Less4)
            & "  OV1=" & SL'image(OV1)
            & "  SAT=" & SL'image(SAT) ;
      else
        assert j = count mod 64
           and s = SAT
           and v = OV1
           and l4 = Less4
          report "mismatch at value "
                & integer'image(count)
                & ":  " & SLV_to_bin(G6)
                & " = " & integer'image(j)  
                & "  EN=" & SL'image(ENABLE)
                & "  L4=" & SL'image(Less4)
                & "  OV1=" & SL'image(OV1)
                & "  SAT=" & SL'image(SAT)
            severity failure;
      end if;
    end show;

  begin
    count := 0;

    -- RESET sequence: pulse CLK once then WR once.
    WR <= '0';
    Clk <= '0' ;
    wait for 200 us;
    Clk <= '1' ;
    wait for 200 us;
    Clk <= '0' ;
    wait for 200 us;
    WR <= '1';
    wait for 200 us;
    WR <= '0';
    wait for 200 us;
    show;

    -- Let's go !
    while count <= maxcount loop
      count := count + 1;
      cycle;
      show;
    end loop;

    -- try some funky reset : reset when CLK high
    count := count + 1;
    wait for 500 us;
    Clk <= '1' ;
    wait for 500 us;
    show;

    WR <= '1';    
    wait for 1000 us;
    count := 0;
    show;
    Clk <= '0' ;
    -- another go
    while count <= maxcount loop
      count := count + 1;
      cycle;
      show;
    end loop;

    -- another funky reset : reset when CLK low
    -- Clk <= '0' ; in cycle()
    wait for 500 us;
    WR <= '0';
    wait for 500 us;
    Clk <= '0' ;
    count := 0;
    show;
    -- another go
    while count <= maxcount loop
      count := count + 1;
      cycle;
      show;
    end loop;
 
    wait;
  end process;

end behavioural;
