-- YGREC8/VHDL/TAP/MUX64/MUX64_BBTBC.vhdl
-- created mar. juil. 21 21:07:06 CEST 2020 by whygee@f-cpu.org
-- version mar. juil. 21 23:01:34 CEST 2020 : classic
-- version mar. juil. 21 23:25:19 CEST 2020 : gates
-- version mer. juil. 22 00:17:06 CEST 2020 : balanced control with sub-MUX16
--
-- Released under the GNU AGPLv3 license
--
-- A better hierarchical MUX64, with 4x MUX16.
-- Fan-in vector is optimal : {1, 12, 12, 12, 13, 13}

Library ieee;
    use ieee.std_logic_1164.all;
Library SLV;
    use SLV.SLV_def.all;
Library proasic3;
    use proasic3.all;
Library work;
	use work.all;

entity MUX64 is
  port(
    leaves : in SLV64;
    CTRL : in SLV6;
    root : out SL);
end MUX64;

architecture gates of MUX64 is
  signal L3 : SLV4;
  signal L2 : SLV2;
  signal s0, s1, s2, s3 : SLV4;

  -- note: reverse naming vs BBT64.svg
  alias a : SL is CTRL(0);
  alias b : SL is CTRL(1);
  alias c : SL is CTRL(2);
  alias d : SL is CTRL(3);
  alias e : SL is CTRL(4);
  alias f : SL is CTRL(5);

begin
  -- the top MUXs :
  l1a: entity MX2  port map(A=>L2(0), B=>L2(1), S=>a, Y=>root);
  l2a: entity MX2  port map(A=>L3(0), B=>L3(1), S=>b, Y=>L2(0));
  l2b: entity MX2  port map(A=>L3(2), B=>L3(3), S=>f, Y=>L2(1));

  -- permutation vector:
  s0 <= f&e&d&c;
  s1 <= d&f&e&c;
  s2 <= b&e&c&d;
  s3 <= c&b&e&d;

  m16a: entity MUX16 port map(leaves=>leaves(15 downto  0), CTRL=>s0, root=> L3(0));
  m16b: entity MUX16 port map(leaves=>leaves(31 downto 16), CTRL=>s1, root=> L3(1));
  m16c: entity MUX16 port map(leaves=>leaves(47 downto 32), CTRL=>s2, root=> L3(2));
  m16d: entity MUX16 port map(leaves=>leaves(63 downto 48), CTRL=>s3, root=> L3(3));
end gates;
