-- YGREC8/VHDL/TAP/MUX64/MUX64_BBTrev.vhdl
-- created mar. juil. 21 21:07:06 CEST 2020 by whygee@f-cpu.org
-- version mar. juil. 21 23:01:34 CEST 2020 : classic
-- version mar. juil. 21 23:25:19 CEST 2020 : gates
-- version mer. juil. 22 00:17:06 CEST 2020 : balanced control with sub-MUX16
-- version mer. juil. 22 15:15:13 CEST 2020 : modified BBTBC
-- version sam. août  1 23:09:49 CEST 2020 : reversed the order of the input bits
-- version 20200806 : synthesis with Synplify
--
-- Released under the GNU AGPLv3 license
--
-- A slightly sub-optimal hierarchical MUX64
-- Fan-in vector is : {1, 11, 11, 11, 14, 15}
-- it's marginally worse but much easier for
-- routing with 4x MUX16-BBTBC. See MUX64-4x16.svg
--
-- SYNTHESIS OK


Library ieee;
    use ieee.std_logic_1164.all;
Library SLV;
    use SLV.SLV_def.all;
Library proasic3;
    use proasic3.all;
Library work;
	use work.all;

entity MUX64 is
  port(
    leaves : in SLV64;
    CTRL : in SLV6;
    root : out SL);
end MUX64;

architecture gates of MUX64 is
  signal L3 : SLV4;
  signal L2 : SLV2;
  signal s0, s1, s2, s3 : SLV4;

  -- note: reverse naming vs BBT64.svg
  alias a : SL is CTRL(5);
  alias b : SL is CTRL(4);
  alias c : SL is CTRL(3);
  alias d : SL is CTRL(2);
  alias e : SL is CTRL(1);
  alias f : SL is CTRL(0);

  component mx2 port(A, B, S : in std_logic; Y : out std_logic); end component;
  component MUX16 port( leaves : in SLV16; CTRL : in SLV4; root : out SL); end component;

begin
  -- the top MUXs :
  l1a: MX2  port map(A=>L2(0), B=>L2(1), S=>f, Y=>root);
  l2a: MX2  port map(A=>L3(0), B=>L3(1), S=>e, Y=>L2(0));
  l2b: MX2  port map(A=>L3(2), B=>L3(3), S=>a, Y=>L2(1));

  -- permutation vector:
  s0 <= a&b&c&d;
  s1 <= a&b&c&d;
  s2 <= e&c&d&b;
  s3 <= e&d&b&c;

  m16a: MUX16 port map(leaves=>leaves(63 downto 48), CTRL=>s0, root=> L3(0));
  m16b: MUX16 port map(leaves=>leaves(47 downto 32), CTRL=>s1, root=> L3(1));
  m16c: MUX16 port map(leaves=>leaves(31 downto 16), CTRL=>s2, root=> L3(2));
  m16d: MUX16 port map(leaves=>leaves(15 downto  0), CTRL=>s3, root=> L3(3));
end gates;
