-- YGREC8/VHDL/TAP/MUX64/test_MUX64.vhdl
-- created mar. juil. 21 21:32:50 CEST 2020 by whygee@f-cpu.org
--
-- Released under the GNU AGPLv3 license
--
-- Scan the 6-bits control code and checks that only 1 input
-- is selected each time.

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library SLV;
    use SLV.SLV_def.all;
Library conversions;
    use conversions.conversions.all;
Library work;
    use work.all;

entity test_MUX64 is
  generic (  verbose: integer := 0 );
  -- top level -> no port
end test_MUX64;

architecture test of test_MUX64 is
  signal leaves : SLV64;
  signal CTRL   : SLV6;
  signal root   : SL;
begin

  dut: entity MUX64 port map(
    leaves => leaves,
      CTRL => CTRL,
      root => root);

  process is
    type t_intarr64 is array(0 to 63) of integer;
    variable histo : t_intarr64 := (others => 0);

    variable count, row : integer;
  begin
    leaves <= (others=>'0');
    for count in histo'range loop
      CTRL <= SLV6(to_unsigned(count, CTRL'length ));

      for row in histo'range loop
        leaves(row) <= '1';
        wait for 1 ms;
        if verbose > 2 then
          report SLV_to_bin(CTRL) & " = " & std_logic'image(root) & " : " &  SLV_to_bin(leaves);
        end if;

        if root = '1' then
          histo(row) := histo(row) + 1;
          if verbose /= 0 then
            report "code " & integer'image(row) 
                  & " => " & integer'image(count);
          end if;
        end if;

        leaves(row) <= '0';
      end loop;

    end loop;

    -- final check :
    assert verbose = 0 report " ** now display the histogram **" severity note;
    for count in histo'range loop
      if verbose = 0 then
        assert histo(count) = 1
          report "code " & integer'image(count)
                & " => " & integer'image(histo(count))
          severity failure;
      else
        report "code " & integer'image(count)
              & " => " & integer'image(histo(count));
      end if;
    end loop;

    wait;
  end process;

end test;

