-- YGREC8/VHDL/TAP/Gray/Gray6s_behavioral.vhdl
-- created ven. août 21 02:55:29 CEST 2020 by whygee@f-cpu.org
--
-- Released under the GNU AGPLv3 license
--
--

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library SLV;
    use SLV.SLV_def.all;
Library conversions;
    use conversions.conversions.all;

entity Selector is
  generic( implemented : SLV6 := "111111" );
  port(
    CLK, WR, SAT, ENABLE, Less4, OV1, preFB, Din : in SL;
    G6 : in SLV6;
    CMD_en : out SL;
    DecA, DecB : out SLV4;
    CMD5 : out SLV5;
    SEL6 : out SLV6);
end Selector;

architecture behavioral of Selector is
  signal IntClk, FirstByte, NoByte, SelB3, SetEN : SL := '0';
  signal CMD_SR : SLV5 := "00000";
  signal SEL_latch, Set_latch : std_logic_vector(6 downto 1) := "000000";

begin

  -- t-latch the clock input :
  IntClk <= CLK when WR='0';

  -- shift the CMD register:
  process (IntClk) is
  begin
    if rising_edge(IntCLK) then
      CMD_SR <= CMD_SR(3 downto 0) & Din;
    end if;
  end process;
  CMD5 <= CMD_SR;

  -- decode the control signals
  NoByte <= not (G6(4) or G6(3) or G6(2));
  SelB3 <= not (Less4 and NoByte and OV1);
  SetEN <= not (IntClk or SelB3);

  FirstByte <= not (G6(4) or G6(3) or IntClk);
  CMD_en <= preFB and Less4 and FirstByte;

  -- More decoders !
  process (CMD_SR(2 downto 0), SetEN) is
    variable i: integer;
  begin
    Set_latch <= (others=>'0');
    i := safe_to_integer(CMD_SR(2 downto 0));
    if SetEN='1' then
      if i < 7 and i > 0 then
--      report "setting Sel bit #" &  integer'image(i);
        Set_latch(i) <= '1';
      end if;
    end if;
  end process;

  process (CMD_SR(1 downto 0)) is
    variable i: integer;
  begin
    i := 0;
    DecA <= (others=>'0');
    if CMD_SR(0) ='1' then i :=   1; end if;
    if CMD_SR(1) ='1' then i := i+2; end if;
    DecA(i) <= '1';
  end process;

  process (CMD_SR(3 downto 2)) is
    variable i: integer;
  begin
    i := 0;
    DecB <= (others=>'0');
    if CMD_SR(2) ='1' then i :=   1; end if;
    if CMD_SR(3) ='1' then i := i+2; end if;
    DecB(i) <= '1';
  end process;

  -- the select RS latches :
  SEL_latch <= (SEL_latch or Set_latch) and (SEL_latch'range => ENABLE);
  SEL6 <= SEL_latch and implemented;

end behavioral;
