-- YGREC8/VHDL/TAP/Gray/Gray6s_tiles.vhdl
-- created ven. août 21 02:55:29 CEST 2020 by whygee@f-cpu.org
-- version sam. août 22 01:17:56 CEST 2020 : tiles version
-- version mar. août 25 05:59:10 CEST 2020 : abstracted SR FF macros added
--
-- Released under the GNU AGPLv3 license

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library SLV;
    use SLV.SLV_def.all;
Library proasic3;
    use proasic3.all;

entity Selector is
  generic( implemented : SLV6 := "111111" );
  port(
    CLK, WR, SAT, ENABLE, Less4, OV1, preFB, Din : in SL;
    G6 : in SLV6;
    CMD_en : out SL;
    DecA, DecB : out SLV4;
    CMD5 : out SLV5;
    SEL6 : out SLV6);
end Selector;

architecture tiles of Selector is
  signal IntClk, FirstByte, NoByte, SelB3, SetEN : SL := '0';
  signal tDecA, tDecB, nCMD : SLV4;
  signal CMD_SR : SLV5 := "00000";
  signal Set_latch : SLV6 := "000000";

  component AND2  port(A, B    : in std_logic; Y : out std_logic); end component;
  component AND3  port(A, B, C : in std_logic; Y : out std_logic); end component;
  component DLN0  port(D,  G   : in std_logic; Q : out std_logic); end component;
  component DFN1  port(D, CLK  : in std_logic; Q : out std_logic); end component;
  component INV   port(A       : in std_logic; Y : out std_logic); end component;
  component NAND3 port(A, B, C : in std_logic; Y : out std_logic); end component;
  component NOR2  port(A, B    : in std_logic; Y : out std_logic); end component;
  component NOR3  port(A, B, C : in std_logic; Y : out std_logic); end component;
  component OA1   port(A, B, C : in std_logic; Y : out std_logic); end component;
  component R0S1  port(S, R    : in std_logic; Y : out std_logic); end component;

begin
  -- t-latch the clock input :
  lck : DLN0 port map(D => CLK, G => WR, Q => IntClk);

  -- the shift register:
  df0: DFN1 port map(D=>Din, CLK=>IntClk, Q=>CMD_SR(0));
  ldf: for i in 1 to 4 generate
    dfn: DFN1 port map(D=>CMD_SR(i-1), CLK=>IntClk, Q=>CMD_SR(i));
    ivn: INV  port map(A=>CMD_SR(i-1), Y => nCMD(i-1));
  end generate;
  CMD5 <= CMD_SR;

  -- decode the control signals
  nby: NOR3  port map(A=>G6(4),  B=>G6(3),  C=>G6(2),  Y=>NoByte);
  sb3: NAND3 port map(A=>Less4,  B=>NoByte, C=>OV1,    Y=>SelB3);
  sen: NOR2  port map(A=>IntClk, B=>SelB3,             Y=>SetEN);

  fby: NOR3  port map(A=>G6(4),  B=>G6(3),  C=>IntClk,  Y=>FirstByte);
  cen: AND3  port map(A=>Less4,  B=>preFB,  C=>FirstByte, Y=>CMD_en);

  -- More decoders !
  dA00: NOR2 port map(A=>CMD_SR(0), B=>CMD_SR(1), Y=>tDecA(0));
  dB00:	NOR2 port map(A=>CMD_SR(2), B=>CMD_SR(3), Y=>tDecB(0));
  dA01: AND2 port map(A=>CMD_SR(0), B=>  nCMD(1), Y=>tDecA(1));
  dB01: AND2 port map(A=>CMD_SR(2), B=>  nCMD(3), Y=>tDecB(1));
  dA10: AND2 port map(A=>  nCMD(0), B=>CMD_SR(1), Y=>tDecA(2));
  dB10:	AND2 port map(A=>  nCMD(2), B=>CMD_SR(3), Y=>tDecB(2));
  dA11: AND2 port map(A=>CMD_SR(0), B=>CMD_SR(1), Y=>tDecA(3));
  dB11: AND2 port map(A=>CMD_SR(2), B=>CMD_SR(3), Y=>tDecB(3));
  DecA <= tDecA;
  DecB <= tDecB;

  lifsp: for i in implemented'range generate
    ifsp: if implemented(i) = '1' generate
      if0: if i<3 generate
        sl012: AND3 port map(A=>tDecA((i+1) mod 4), B=>nCMD(2),   C=>SetEN, Y=>Set_latch(i));
      end generate;
      if1: if i>2 generate
        sl345: AND3 port map(A=>tDecA((i+1) mod 4), B=>CMD_SR(2), C=>SetEN, Y=>Set_latch(i));
      end generate;
      dl: R0S1  port map(S=>Set_latch(i), R=>ENABLE, Y=>SEL6(i));
    end generate;
    -- "else"
    ifsn: if implemented(i) /= '1' generate
      SEL6(i) <= '0';
    end generate;
  end generate;

end tiles;
