-- YGREC8/VHDL/TAP/Selector/test_Selector.vhdl
-- created ven. août 21 21:44:11 CEST 2020 by whygee@f-cpu.org
--
-- Released under the GNU AGPLv3 license
--
-- Check the behaviour of the TAP's Selector unit.

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library SLV;
    use SLV.SLV_def.all;
Library conversions;
    use conversions.conversions.all;
Library work;
    use work.all;

entity test_Selector is
  generic (  verbose: integer := 0 );
  -- top level -> no port
end test_Selector;

architecture simple of test_Selector is
  signal WR, Din, ENABLE, preFB, FB,
     CMD_en, SAT, Less4, OV1 : SL := '0';
  signal CLK : SL := '1';
  signal DecA, DecB : SLV4;
  signal CMD5 : SLV5;
  signal G6, SEL6 : SLV6;

  component Gray6s port(
    CLK, WR : in  SL;
    SAT, ENABLE, preFB, FB, Less4, OV1 : out SL;
    G6 : out SLV6);
  end component;

  component Selector generic( implemented : SLV6 );
  port(
    CLK, WR, SAT, ENABLE, Less4, OV1, preFB, Din : in SL;
    G6 : in SLV6;
    CMD_en : out SL;
    DecA, DecB : out SLV4;
    CMD5 : out SLV5;
    SEL6 : out SLV6);
  end component;

begin

  gr: Gray6s port map(
      Clk => CLK,
       WR => WR,
      SAT => SAT,
   ENABLE => ENABLE,
    preFB => preFB,
       FB => FB,
      OV1 => OV1,
    Less4 => Less4,
       G6 => G6 );

  dut: Selector generic map(implemented => "111111")
    port map(
      Din => Din,
      Clk => CLK,
       WR => WR,
      SAT => SAT,
   ENABLE => ENABLE,
    preFB => preFB,
      OV1 => OV1,
    Less4 => Less4,
       G6 => G6,
   CMD_en => CMD_en,
     DecA => DecA,
     DecB => DecB,
     CMD5 => CMD5,
     SEL6 => SEL6);

  process is
    procedure shift_bit(b : SL) is
    begin
      Din <= b;
--      if verbose > 1 then
--        report "Din=" & SL'image(b);
--      end if;
      CLK <= '0';
      wait for 1 ms; 
      CLK <= '1';
      wait for 1 ms; 
    end procedure;

    procedure shift_in(I : std_logic_vector) is
      variable l : integer;
    begin  -- MSB first !
      for l in I'range loop
--        report "bit #" & integer'image(l);
        shift_bit(SL(I(l)));
      end loop;
      -- don't forget to reset the CLK before risign WR !
      CLK <= '0';
      wait for 1 ms; 
    end procedure;

    variable V : SLV5 := "10011";
    variable exp6 : SLV6 := "000000";
    variable N3 : unsigned(2 downto 0);

  begin
    CLK <= '1';  -- prevent the " simulation stopped by --stop-delta " error.

    -- "reset" dance :
    WR <= '0';
    wait for 1 ms;
    CLK <= '0';
    wait for 1 ms;
    CLK <= '1' ;
    wait for 200 us;
    CLK <= '0' ;
    wait for 200 us;
    WR <= '1';
    wait for 200 us;
    WR <= '0';
    wait for 200 us;

    -- test the shift register :
    for i in V'range loop
      shift_bit(V(i));
--      report "loop " & integer'image(i);
--           & "SR = " & SLV_to_bin(CMD5);
--           & "cmp: " & SLV_to_bin(CMD5(4-i downto 0))
--           & "  V: " & SLV_to_bin(V(4 downto i));
      assert CMD5(4-i downto 0) = V(4 downto i)
          report "Shift register output mismatch" severity failure;
    end loop;

    -- test the inhibit of the shift register
    WR <= '1';
    wait for 1 ms;
      shift_bit('0');
      assert CMD5 = V report "Shift register mismatch (no inhibit ?)" severity failure;
    WR <= '0';
    wait for 1 ms;

    -- the counter is now cleared again : test the CMD_en output
    for i in 1 to 80 loop
      shift_bit('0');
      CLK <= '0';    -- CMD is active on CLK low
      wait for 1 ms;

--      report "loop " & integer'image(i) & " : CMD_en=" & SL'image(CMD_en)
--         & "  preFB="     & SL'image(preFB) 
--         & "  Less4="     & SL'image(Less4)
--         & "  G6=" & SLV_to_bin(G6);

      if i=7 then
        assert CMD_en='1' report "CMD_en output mismatch" severity failure;
      else
        assert CMD_en='0' report "CMD_en output mismatch" severity failure;
      end if;

    end loop;

    -- clear again...
    WR <= '1';
    wait for 1 ms;
    WR <= '0';
    wait for 1 ms;

    -- test the predecoded outputs
    exp6 := "000000";
    for i in 0 to 3 loop
      N3 := to_unsigned(i, 3); -- we only use 2 bits though
      shift_in(std_logic_vector(N3(1 downto 0))
             & std_logic_vector(N3(1 downto 0)));
      exp6(i) := '1';

--	report "loop " & integer'image(i)
--             & "  SR=" & SLV_to_bin(CMD5(3 downto 0))
--             & "  DecA=" & SLV_to_bin(DecA)
--             & "  DecB=" & SLV_to_bin(DecB)
--             & "  exp=" & SLV_to_bin(exp6(3 downto 0)) ;

      assert exp6(3 downto 0) = DecA
          report "DecA mismatch : found " & SLV_to_bin(DecA)
              & " vs " & SLV_to_bin(exp6(3 downto 0));
      assert exp6(3 downto 0) = DecB
          report "DecB mismatch : found " & SLV_to_bin(DecB)
              & " vs " & SLV_to_bin(exp6(3 downto 0));

      exp6(i) := '0';
    end loop;

    -- test the selected outputs
    for i in exp6'range loop
      -- clear again...
      WR <= '1';
      wait for 1 ms;
      WR <= '0';
      wait for 1 ms;
      assert SEL6="000000";
      N3 := to_unsigned(i+1, 3);
      shift_in(std_logic_vector(N3 & "00"));
      CLK <= '0';    -- output is active on CLK low
      wait for 1 ms;
--      report "  loop " & integer'image(i+1)
--           & "  - N3=" & SLV_to_bin(SLV3(N3))
--           & "  SEL6=" & SLV_to_bin(SEL6)
--           & "    SR=" & SLV_to_bin(CMD5) ;
      exp6(i) := '1';
      assert exp6 = SEL6 report " SEL6 output mismatch" severity failure;
      exp6(i) := '0';
    end loop;

    wait;
  end process;

end simple;
