-- YGREC8/VHDL/TAP/TAP.vhdl
-- created mer. juil. 22 20:04:42 CEST 2020 by whygee@f-cpu.org
--
-- Released under the GNU AGPLv3 license
--
-- serialises the 64 "leaves" input bits to SDO (MSB first)
-- under the control of the /WR and CLK pins.

Library ieee;
    use ieee.std_logic_1164.all;
Library SLV;
    use SLV.SLV_def.all;
Library work;
    use work.all;

entity TAP is
    -- is gating the MUX64 input providing
    -- a significant reduction of activity or power ?
    -- is it worth the 6 added gates ?
  generic (mux_gated: integer := 1);
    -- 0 is direct, 1 is ANDed with the WR pin

  port( leaves : in SLV64;
        WR, CLK, Din : in SL;
--        ToMem, Instr, FromMem, SL8 : out SLV8;
        Dout : out SL);
end TAP;

architecture simple of TAP is
  -- Flags from the Gray counter
  signal ENABLE, preFB, FB, SAT, Less4, OV1 : SL;
  -- link between the Gray counter and the MUX tree
  signal G6, G6M : SLV6;

  component Gray6s port(
    CLK, WR : in  SL;
    SAT, ENABLE, preFB, FB, Less4, OV1 : out SL;
    G6 : out SLV6);
  end component;

--  component AND2  port(A, B : in std_logic; Y : out std_logic); end component;

  component MUX64 port(
    leaves : in SLV64;
    CTRL : in SLV6;
    root : out SL);
  end component;

begin

  -- The core of the serialiser / PISO

  gr: Gray6s port map(
      Clk => CLK,
       WR => WR,
      SAT => SAT,
   ENABLE => ENABLE,
    preFB => preFB,
       FB => FB,
      OV1 => OV1,
    Less4 => Less4,
       G6 => G6 );

  l1: if mux_gated=0 generate
    G6M <= G6;
  end generate;
  l2: if mux_gated/=0 generate
    G6M <= G6 when WR='1' else (G6'range=>'0');
  end generate;

  mx: MUX64 port map(
    leaves => leaves,
      CTRL => G6M,
      root => Dout);

end simple;
