-- YGREC8/VHDL/TAP/MUX64/test_PISO_order.vhdl
-- created mer. juil. 22 21:30:53 CEST 2020 by whygee@f-cpu.org
-- version dim. août 16 02:03:23 CEST 2020
--
-- Released under the GNU AGPLv3 license
--
-- Exercises the serialiser and (eventually) outputs the permutation vector.

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library std;
    use std.textio.all;
Library SLV;
    use SLV.SLV_def.all;
Library read_integers;
    use read_integers.read_integers.all;  -- defines t_intarr
Library work;
    use work.all;

entity PISO_order is
  generic (  verbose: integer := 0 );
   -- verbose = 1 => outputs individual values
   -- verbose = 2 => outputs a string for inclusion in other programs.

  -- top level -> no port
end PISO_order;


architecture test of PISO_order is
  signal leaves : SLV64;
  signal WR, CLK, Din, Dout : SL := '0';
begin

  dut: entity TAP port map(
    leaves => leaves,
        WR => WR,
       CLK => CLK,
       Din => Din,
      Dout => Dout);

  process is
    variable dirmap, revmap : t_intarr(0 to leaves'length-1) := (others => 0);
    variable count, row, sel : integer;
    variable l:line;
  begin
    leaves <= (others=>'0');

    -- RESET sequence: pulse CLK once then WR once.
    WR <= '0';
    Clk <= '0' ;
    wait for 200 us;
    Clk <= '1' ;
    wait for 200 us;
    Clk <= '0' ;
    wait for 200 us;
    WR <= '1';
    wait for 200 us;
    WR <= '0';
    wait for 200 us;

    -- but this test of the MUX requires to read it so
    WR <= '1';

    for count in revmap'range loop
      sel := 0;
      for row in revmap'range loop
        leaves(row) <= '1';
        wait for 1 ms;

        if Dout = '1' then
          sel := sel + 1;
          assert verbose /= 1
            report "count#"    & integer'image(count)
                & " => leaf #" & integer'image(row)
            severity note;

          assert dirmap( row ) = 0
            report " Duplicate signal for count#" & integer'image(count)
                & " => leaf #" & integer'image(row)
                 severity failure;
          dirmap( row ) := count;

          assert revmap(count) = 0
            report " Duplicate signal for count#" & integer'image(count)
                & " => leaf #" & integer'image(row)
                 severity failure;
          revmap(count) := row;
        end if;

        leaves(row) <= '0';
      end loop;

      assert sel = 1
        report "count#" & integer'image(count)
          & " has sel=" & integer'image(sel)
           severity failure;

     -- get the next bit
     clk <= '1';
     wait for 500 us;
     clk <= '0';
     wait for 500 us;
    end loop;

    if verbose = 2 then
      for count in 0 to revmap'high-1 loop
        write(l, integer'image(revmap(count)) & ", "); 
      end loop;
      write(l, integer'image(revmap(revmap'high)));
      writeline(OUTPUT, l);
    end if;

    -- not used as expected, but code is kept anyway :
    if verbose = 3 then
      for count in 0 to revmap'high-1 loop
        write(l, integer'image(dirmap(count)) & ", "); 
      end loop;
      write(l, integer'image(dirmap(revmap'high)));
      writeline(OUTPUT, l);
    end if;

    wait;
  end process;

end test;
