-- file : test_readback.vhdl
-- created jeu. juil. 23 09:18:50 CEST 2020
-- version dim. août 16 02:17:00 CEST 2020
--
-- read the permutation vector from a file
-- then inject and read back data through the TAP
-- to check if bits are correctly descrambled


library ieee;
    use ieee.std_logic_1164.all;
library std;
        use std.textio.all;
Library SLV;
    use SLV.SLV_def.all;
Library read_integers;
    use read_integers.read_integers.all; 
Library conversions;
    use conversions.conversions.all;
Library work;
    use work.all;

entity test_readback is
end test_readback;

architecture test of test_readback is
  signal value64, tmp64 : std_ulogic_vector(63 downto 0) := (others=>'0');
  signal WR, Dout, Din, result, clk_l, clk_t, reset : SL := '0';

  constant revmap : t_intarr(0 to 63) := read_integer("PISO_permutation.out", 64);

begin

  process
    variable v64 : SLV64;
    variable i, j, count : integer;
--    variable l : line;
  begin
    WR <= '0';
    clk_t <= '0';
    clk_l <= '0';
    reset <= '0';

    wait for 1 ms; -- Why does it take 4s with the TILES version of Gray6s ?? 

    -- Init the LFSR
    reset <= '1';

    -- RESET sequence: pulse CLK once then WR once.
    clk_t <= '1' ;
    wait for 200 us;
    clk_t <= '0' ;
    wait for 200 us;
    WR <= '1';
    wait for 200 us;
    WR <= '0';
    wait for 200 us;

    for i in 1 to 31 loop -- could be any duration but that should be enough
      -- scramble the bits again :
      for j in 1 to 13 loop
        clk_l <= '1'; wait for 1 ms;
        clk_l <= '0'; wait for 1 ms;
      end loop;
--      write(l, SLV_to_bin(std_logic_vector(value64)) & " : " );

      wr <= '1';  -- enable the Gray counter
      wait for 1 ms;

      v64 := (others=>'X');
      for j in revmap'range loop
        assert Dout = value64(revmap(j))
          report "mismatch at bit #" & integer'image(j)
          severity failure;

--        v64(j) := Dout xor value64(revmap(j));  -- must be 0
--  or:
        v64(revmap(j)) := Dout;
        clk_t <= '1'; wait for 1 ms;
        clk_t <= '0'; wait for 1 ms;
      end loop;

      assert v64 = std_logic_vector(value64)
          report "mismatch : " & SLV_to_bin(v64) & 
               " /= " & SLV_to_bin(std_logic_vector(value64))
          severity failure;

      wr <= '0';
      wait for 1 ms;
--      write(l, SLV_to_bin(v64 xor std_logic_vector(value64))); must show 0s
--      writeline(output,l);
    end loop;

    wait;
  end process;

  gen: entity work.lfsr_64
    port map (
           clk => clk_l,
         reset => reset,
           din => '0',
          lfsr => value64,
      init_val => "1010010110101010011011101000100011010010110101100100101010011011");
-- why have it easy ? :-P
-- bin=""; od -An -v -w1 -t x1 /dev/urandom |sed 's/[ 2-9a-f]//g'| \
-- while read line; do bin="$bin$line" ; echo $bin; \
-- [ $(echo -n $bin |wc -c) -gt "63" ] && break; done

  -- latching/gating the value to prevent useless activity during LFSR scrambling
  tmp64 <= value64 when WR='1';

  dut: entity TAP port map(
    leaves => std_logic_vector(tmp64),
        WR => WR,
       CLK => clk_t,
       Din => Din,
      Dout => Dout);

end test;
