-- file : MISC/lfsr_64.vhdl
-- derived from lfsr4.vhdl (parameterised LFSR generator)
--  version dim. août 16 02:53:28 CEST 2020
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- This is a non-parameterised version that uses less room/time to use.

library ieee;
use ieee.std_logic_1164.all;

entity lfsr_64 is
  port(
    clk, reset, din : in std_logic := '0';
    lfsr  : inout std_ulogic_vector(64 downto 1);
    init_val : in std_ulogic_vector(64 downto 1)
         := (1=>'1', others=>'0')); -- leave "open" or connect to a constant only
end lfsr_64;

architecture fibonacci of lfsr_64 is
begin
  process (clk, reset)
  begin
    if reset /= '1' then
      lfsr <= init_val;
    else
      if rising_edge(clk) then
        lfsr(1) <= lfsr(64)
               xor lfsr(63)
               xor lfsr(61)
               xor lfsr(60)
               xor din;
        lfsr(64 downto 2) <= lfsr(63 downto 1);
      end if;
    end if;
  end process;
end fibonacci;
