-- read_integers.vhdl
-- created jeu. juil. 23 11:53:17 CEST 2020
-- just initialises an array of integers with the contents of a file,
-- such that a constant array can be read at elaboration time.

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
library std;
    use std.textio.all;
Library conversions;
    use conversions.conversions.all;

package read_integers is
  type t_intarr is array(integer range <>) of integer;
  impure function read_integer(filename: string; size: integer) return t_intarr;
end read_integers;

package body read_integers is
  impure function read_integer(filename: string; size: integer) return t_intarr is
    type char_file is file of character;
    file permutation_file: char_file open read_mode is filename;
    variable arr: t_intarr(0 to size-1);
    variable i, j, len, val, diag : integer := 0;
    variable s : string (1 to 12) := "42          ";
    variable c : character;

  begin
--    report "reading " & integer'image(size) & " integers";

    for i in arr'range loop

      -- skip separators: look for a character 0-F
      loop
        assert not endfile(permutation_file)
          report "early end of input file " & filename
            & " after " & integer'image(i) & " values."
          severity failure;
        read(permutation_file, c);
        exit when base_lookup(c) >= 0;
      end loop;

      -- now get as many characters as possible
      len := 1;
      loop
        s(len) := c;
        exit when endfile(permutation_file);
        read(permutation_file, c);
        exit when base_lookup(c) = -1;
        len := len+1;
        assert len <= s'high
          report "number is too large ! " & s
            severity failure;
      end loop;

      val := 0;
      diag := 0;
      Num_to_int_p(s(1 to len), val, diag);

      assert diag=0
        report "conversion error for : " & s(1 to len)
        severity failure;

      arr(i) := val;
    end loop;
    file_close(permutation_file);
--    report "read ok";
    return arr;
  end read_integer;

end read_integers;
